
program SHTM_3D
    use G_globals
  implicit none

! == Local Parameters and Data Declaration
! == Local Variable Declaration and Description
integer ix,iy,iz,OutNtot

real Esond, Etip
real, dimension (3) :: Rtip,Rsond, Fsond, Rtmp, Rsond0, Ftip

integer ipos,npos
real, dimension (:,:), allocatable :: poslist

real, dimension (3) :: a1,a2
real R1, T1,T2, TSS1, TSS2

real time_start,time_end
real time_start2,time_end2

! == Procedure

a1 = (/ 0,0,1 /)  ! orientation of pz orbitals
! beta = 1.0


write (*,*) " >> Reading SHTM.ini"
  open (unit = 69, file = "SHTM_3D.ini", status = 'old')
  read (69,*) wrtDebug
  read (69,*) format_Out
  read (69,*) relaxMethod
  read (69,*) dt, convergF, damping, startKick, maxRelaxIter  
  read (69,*) kMorse 
  read (69,*) beta1, beta2
  read (69,*) kHarmonic
  read (69,*) RspringMin
  read (69,*) sondZ
  read (69,*) OutE%step(:)
  read (69,*) OutE%Rmin(:)
  read (69,*) OutE%Rmax(:)
  close (69)



  ! Grids over Tip position
write (*,*         ) " >> Initialize Grids over Tip position "
  call OutE%initgrid()
  call OutE%echoSetup()
  call OutFx%copySetup(OutE)
  call OutFy%copySetup(OutE)
  call OutFz%copySetup(OutE)
  call OutX%copySetup(OutE)
  call OutY%copySetup(OutE)
  call OutZ%copySetup(OutE)
  call OutETip%copySetup(OutE)
  call OutFzTip%copySetup(OutE)
  call OutT1%copySetup(OutE)
  call OutT2%copySetup(OutE)
  !call OutT12%copySetup(OutE)
  call OutTSS1%copySetup(OutE)
  call OutTSS2%copySetup(OutE)
  call OutR1%copySetup(OutE)
  OutNtot = OutE%N(1)*OutE%N(2)*OutE%N(3) 

  ! atomic parameters
write (*,*) " >> Read atomic positions and parameters "
call readspecies()
call surf%fromfile( 'surf.bas' )
call tip%fromfile ( 'tip.bas'  )

open (unit = 69, file = "poslist.ini", status = 'old')
	read (69,*) npos
	allocate( poslist(3,npos) )
	do ipos = 1,npos
		read (69,*) Rtmp(:)
		poslist(1,ipos) =  (  atypes( tip%Zs(1) )%R0 + atypes( SondZ )%R0  ) * Rtmp(1) * sin( 2*3.14159265359*Rtmp(2) ) * cos( 2*3.14159265359*Rtmp(3) )
		poslist(2,ipos) =  (  atypes( tip%Zs(1) )%R0 + atypes( SondZ )%R0  ) * Rtmp(1) * sin( 2*3.14159265359*Rtmp(2) ) * sin( 2*3.14159265359*Rtmp(3) )
		poslist(3,ipos) = -(  atypes( tip%Zs(1) )%R0 + atypes( SondZ )%R0  ) * Rtmp(1) * cos( 2*3.14159265359*Rtmp(2) )  
		if ( wrtDebug .gt. 0 ) write(*,'(A,i5,6f20.10)') " i, R,theta,phi, pos0 x,y,z: ", ipos,Rtmp(:), poslist(:,ipos)
	end do ! ipos

Rsond0  = 0
Rsond0(3) =  - ( atypes( tip%Zs(1) )%R0 + atypes( SondZ )%R0 )
write (*,'(A,3f20.5)')  " Rsond0 normalized :   ",  Rsond0 



write (*,*) " >> TIP sampling .... "
      OutE %f = 0
      OutFx%f = 0
      OutFy%f = 0
      OutFz%f = 0
      OutX%f = 0
      OutY%f = 0
      OutZ%f = 0
      OutEtip %f = 0
      OutFzTip%f = 0
      OutT1%f = 0
      OutT2%f = 0
      !OutT12%f = 0
      OutTSS1%f = 0
      OutTSS2%f = 0
      OutR1%f   = 0
call cpu_time(time_start2)

Rtip(1) = OutE%Rmin(1)
do ix = 1, OutE%N(1)
	Rtip(2) = OutE%Rmin(2)
	do iy = 1, OutE%N(2)
		Rtmp(:) = Rtip(:) - tip%Rs(:,1)                                  ! move tip to pos first atom to Rtip
		call tip%move( Rtmp )
		Rsond(:) = tip%Rs(:,1) + Rsond0
		write (*,'(A,2i5)') " ix, iy   ",ix,iy 
    	do iz = 1, OutE%N(3)
			Rtmp(:) = Rtip(:) - tip%Rs(:,1)                                  ! move tip to pos first atom to Rtip
			call tip%move( Rtmp )
			Etip = 0.0
			Ftip = 0.0
			call getFF_LJ  ( tip%Rs(:,1), tip%Zs(1), surf, Etip, Ftip )   ! direct TipSurface Interaction
			iiter = 0
			if (relaxMethod .gt. 0) then
				!Rsond(:) = tip%Rs(:,1) + Rsond0(:)
				Fsond(:) = 0
				Esond    = 0
				call getFF_LJ       ( Rsond, sondZ, surf, Esond, Fsond)  ! Surface potential
				!call getFF_LJ       ( Rsond, sondZ,    tip, Esond, Fsond)  ! Tip poteitnal
				if (Esond .gt. 0) then
					do ipos = 1,npos
						Rsond(:) = tip%Rs(:,1) + poslist(:,ipos)
						Fsond(:) = 0
						Esond    = 0
						call getFF_LJ       ( Rsond, sondZ, surf, Esond, Fsond)  ! Surface potential
						call getFF_LJ       ( Rsond, sondZ,  tip, Esond, Fsond)  ! Tip poteitnal
						if (Esond .lt. 0) exit
					end do! ipos
				end if ! Esond
				call relax( surf, Rtip,  Rsond, Fsond, Esond )  ! this is the importaint
			else
				!write (*,*) "  fixed "
				Rsond(:) = tip%Rs(:,1) + Rsond0(:)
				Fsond(:) = 0
				Esond    = 0
				call getFF_LJ  ( Rsond, sondZ, surf, Esond, Fsond )
			end if
			a2(:) = Rsond(:) - tip%Rs(:,1)
			R1 = sqrt(dot_product(a2,a2))
			a2(:) = a2(:)/R1
			call getHoppingPP ( beta1, Rsond, a1, a2,  tip, T1 )
			call getHoppingPP ( beta2, Rsond, a1, a2,  surf, T2 )
			call getHoppingSS ( beta1, Rsond, tip, TSS1 )
			call getHoppingSS ( beta2, Rsond, surf, TSS2 )
			if (wrtDebug .gt. 0) write (*,'(A,4i5,4f25.10)') " ix,iy,iz,iters,E,Fz,RzTip,RzSond ",ix,iy,iz, iiter, Rtip(3), Rsond(3), Esond, Fsond(3)
      		OutE %f(ix,iy,iz) = Esond
      		OutFx%f(ix,iy,iz) = Fsond(1)
      		OutFy%f(ix,iy,iz) = Fsond(2)
      		OutFz%f(ix,iy,iz) = Fsond(3)
      		OutX%f(ix,iy,iz)  = Rsond(1)-Rtip(1)
      		OutY%f(ix,iy,iz)  = Rsond(2)-Rtip(2)
      		OutZ%f(ix,iy,iz)  = Rsond(3)-Rtip(3)
      		OutETip %f(ix,iy,iz) = Esond + Etip
      		OutFzTip%f(ix,iy,iz) = Fsond(3) + Ftip(3) 
      		OutT1%f(ix,iy,iz)  = T1
      		OutT2%f(ix,iy,iz)  = T2
      		!OutT12%f(ix,iy,iz) = T1*T2
      		OutTSS1%f(ix,iy,iz) = TSS1
			OutTSS2%f(ix,iy,iz) = TSS2
			OutR1%f(ix,iy,iz)   = R1
			Rtip(3) = Rtip(3) - OutE%step(3)
		end do ! iz
		Rtip(2) = Rtip(2) + OutE%step(2)
		Rtip(3) = OutE%Rmax(3)
    end do ! iy
	Rtip(1) = Rtip(1) + OutE%step(1)
end do ! ix

call cpu_time(time_end2)
 close(70) 
 close(71) 
write (*,'(A,f10.5,A)') " >>  .... done in  ", (time_end2-time_start2) ,"[sec]"
write (*,'(A,f10.5,A)') " >>  = ", (time_end2-time_start2)/OutNtot,"[sec] per tip step "

call OutE %output("OutE" ,format_Out,1.0)
call OutFx%output("OutFx",format_Out,1.0)
call OutFy%output("OutFy",format_Out,1.0)
call OutFz%output("OutFz",format_Out,1.0)
call OutX%output("OutX",format_Out,1.0)
call OutY%output("OutY",format_Out,1.0)
call OutZ%output("OutZ",format_Out,1.0)

call OutETip %output("OutETip" ,format_Out,1.0)
call OutFzTip%output("OutFzTip",format_Out,1.0)

call OutT1%output("OutT1",format_Out,1.0)
call OutT2%output("OutT2",format_Out,1.0)
!call OutT12%output("OutT12",format_Out,1.0)

call OutTSS1%output("OutTSS1",format_Out,1.0)
call OutTSS2%output("OutTSS2",format_Out,1.0)
call OutR1%output  ("OutR1",format_Out,1.0)

stop

end program SHTM_3D
