
program SHTM_3D
    use G_globals
  implicit none

! == Local Parameters and Data Declaration
! == Local Variable Declaration and Description
integer ix,iy,iz,OutNtot

real Esond
real, dimension (3) :: Rtip,Rsond, Fsond

real time_start,time_end
real time_start2,time_end2

! == Procedure

write (*,*) " >> Reading SHTM.ini"
  open (unit = 69, file = "SHTM_3D.ini", status = 'old')
  read (69,*) wrtDebug
  read (69,*) format_Out
  read (69,*) relaxMethod
  read (69,*) dt, convergF, damping, startKick, maxRelaxIter  
  read (69,*) kMorse 
  read (69,*) kHarmonic
  read (69,*) sondZ
  read (69,*) OutE%step(:)
  read (69,*) OutE%Rmin(:)
  read (69,*) OutE%Rmax(:)
  close (69)


  ! Grids over Tip position
write (*,*         ) " >> Initialize Grids over Tip position "
  call OutE%initgrid()
  call OutE%echoSetup()
  call OutFx%copySetup(OutE)
  call OutFy%copySetup(OutE)
  call OutFz%copySetup(OutE)
  call OutX%copySetup(OutE)
  call OutY%copySetup(OutE)
  call OutZ%copySetup(OutE)
  OutNtot = OutE%N(1)*OutE%N(2)*OutE%N(3) 

  ! atomic parameters
write (*,*) " >> Read atomic positions and parameters "
call readspecies()
call surf%fromfile('surf.bas')

write (*,*) " >> TIP sampling .... "
      OutE %f = 0
      OutFx%f = 0
      OutFy%f = 0
      OutFz%f = 0
      OutX%f = 0
      OutY%f = 0
      OutZ%f = 0
call cpu_time(time_start2)

Rtip(1) = OutE%Rmin(1)
do ix = 1, OutE%N(1)
	Rtip(2) = OutE%Rmin(2)
	do iy = 1, OutE%N(2)
		Rsond = Rtip
		write (*,'(A,2i5)') " ix, iy   ",ix,iy 
    	do iz = 1, OutE%N(3)
			if (relaxMethod .gt. 0) then
				call relax( surf, Rtip,  Rsond, Fsond, Esond )  ! this is the importaint
			else
				Rsond(:) = Rtip(:)
				Fsond(:) = 0
				Esond    = 0
				call getFF_LJ  ( Rsond, sondZ, surf, Esond, Fsond )
			end if
			if (wrtDebug .gt. 0) write (*,'(A,4i5,4f25.10)') " ix,iy,iz,iters,E,Fz,RzTip,RzSond ",ix,iy,iz, iiter, Rtip(3), Rsond(3), Esond, Fsond(3)
      		OutE %f(ix,iy,iz) = Esond
      		OutFx%f(ix,iy,iz) = Fsond(1)
      		OutFy%f(ix,iy,iz) = Fsond(2)
      		OutFz%f(ix,iy,iz) = Fsond(3)
      		OutX%f(ix,iy,iz)  = Rsond(1)-Rtip(1)
      		OutY%f(ix,iy,iz)  = Rsond(2)-Rtip(2)
      		OutZ%f(ix,iy,iz)  = Rsond(3)-Rtip(3)
			Rtip(3) = Rtip(3) - OutE%step(3)
		end do ! iz
		Rtip(2) = Rtip(2) + OutE%step(2)
		Rtip(3) = OutE%Rmax(3)
    end do ! iy
	Rtip(1) = Rtip(1) + OutE%step(1)
end do ! ix

call cpu_time(time_end2)
 close(70) 
 close(71) 
write (*,'(A,f10.5,A)') " >>  .... done in  ", (time_end2-time_start2) ,"[sec]"
write (*,'(A,f10.5,A)') " >>  = ", (time_end2-time_start2)/OutNtot,"[sec] per tip step "

call OutE %output("OutE" ,format_Out,1.0)
call OutFx%output("OutFx",format_Out,1.0)
call OutFy%output("OutFy",format_Out,1.0)
call OutFz%output("OutFz",format_Out,1.0)
call OutX%output("OutX",format_Out,1.0)
call OutY%output("OutY",format_Out,1.0)
call OutZ%output("OutZ",format_Out,1.0)

stop

end program SHTM_3D
