

subroutine move_simple( R, v, F)
	use G_globals
 implicit none
! == Local Parameters and Data Declaration
	real, dimension (3), intent(inout)  :: R
	real, dimension (3), intent(inout)  :: v
	real, dimension (3), intent(in)     :: F
! == Procedure
	v = v + F*dt
	v = v*(1.0-damping)
	R = R + v*dt
end subroutine move_simple

subroutine move_FIRE_1( R, v, F)
	use G_globals
 implicit none
! == Local Parameters and Data Declaration
	real, dimension (3), intent(inout)  :: R
	real, dimension (3), intent(inout)  :: v
	real, dimension (3), intent(in)     :: F
! == Local variables
	real, dimension (3)   :: vhat, Fhat
	real                  :: vlen
! == Procedure
	Fhat = F / ( sqrt(dot_product(F,F)) + 0.1e-12 )
	vlen = ( sqrt(dot_product(v,v)) + 0.1e-12 )
	vhat = v / vlen
	v = v + F*dt
	v = v - damping*vlen*( vhat - Fhat )
	R = R + v*dt
	! write (*,'(A,10f10.5)') " dt,v,F,Fhat", dt,v,F,Fhat
end subroutine move_FIRE_1

subroutine relax( system,  Rtip,   Rsond, Fsond, Esond  )
	use G_globals
 implicit none
! == Local Parameters and Data Declaration
  	type (subSystem)   , intent(in)     :: system
	real, dimension (3), intent(in)     :: Rtip
	real, dimension (3), intent(inout)  :: Rsond
	real, dimension (3), intent(out)    :: Fsond
	real, intent(out)                   :: Esond
! == Local variables
	integer i
	real, dimension (3)   :: v, RTipSond
	real                  :: Etemp
! == Procedure
	if (startKick .gt. 0) then
		Esond = 0
		Fsond(:) = 0.D0
		call getFF_LJ       ( Rsond, sondZ, system, Esond, Fsond)  ! Surface potential
		call getFF_LJ       ( Rsond, sondZ,   tip, Esond, Fsond)  ! Tip poteitnal
		RTipSond = Rsond - Rtip - RspringMin
		call getFF_Harmonic ( RTipSond,  Esond, Fsond)  ! Tip poteitnal
		v = startKick*dt*Fsond
	else
		v(:) = 0
	end if
	do iiter = 1, maxRelaxIter
		Esond = 0
		Fsond(:) = 0.D0
		call getFF_LJ       ( Rsond, sondZ, system, Esond, Fsond)  ! Surface potential
		call getFF_LJ       ( Rsond, sondZ,    tip, Esond, Fsond)  ! Tip poteitnal
		RTipSond = Rsond - Rtip - RspringMin
		! write (*,'(A,i5,7f10.5)') " before ",iiter,Esond, Fsond, Rsond
		call getFF_Harmonic ( RTipSond,  Esond, Fsond)      ! Tip poteitnal
		! write (*,'(A,i5,7f10.5)') " after ",iiter,Esond, Fsond, Rsond
		if (relaxMethod .eq. 1) call move_simple(Rsond,v,Fsond)
		if (relaxMethod .eq. 2) call move_FIRE_1(Rsond,v,Fsond)
		if ( wrtDebug .gt. 1 ) write (*,'(A,i5,f25.5,5x, 3f25.5,5x, 3f25.5 )') 'i,E,Fx,Fy,Fz,Rx,Ry,Rz ',i, Esond, Fsond, Rsond
		if (dot_product(Fsond,Fsond) .lt. convergF**2) exit
		if (wrtDebug .gt. 2) then
			write (2000,'(i5)')   (1+tip%n + surf%n) 
			write (2000,'(A,i5,3f20.10)') "  iz, iteration ", iiter, Rtip
			write (2000, '( A, 3f20.8 )') atypes(sondZ)%symbol, Rsond
			call system2XYZ ( tip, 2000)
			call system2XYZ ( surf, 2000)
		end if ! wrtDebug 
	end do
	Fsond(:) = 0.D0
	call getFF_LJ  ( Rsond, sondZ, system, Etemp, Fsond )
	if (iiter .ge. maxRelaxIter ) then
		write (*,*) " Not converged in ",maxRelaxIter,' steps '
	end if
end  subroutine relax
